### ms-wkst.idl
"""
RPC definitions for the following interfaces:
- wkssvc (v1.0): 6BFFD098-A112-3610-9833-46C3F87E345A
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRUnionField,
    register_dcerpc_interface,
)


class LPWKSTA_INFO_100(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("wki100_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki100_computername", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki100_langroup", ""), deferred=True
        ),
        NDRIntField("wki100_ver_major", 0),
        NDRIntField("wki100_ver_minor", 0),
    ]


class LPWKSTA_INFO_101(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("wki101_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki101_computername", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki101_langroup", ""), deferred=True
        ),
        NDRIntField("wki101_ver_major", 0),
        NDRIntField("wki101_ver_minor", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki101_lanroot", ""), deferred=True
        ),
    ]


class LPWKSTA_INFO_102(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("wki102_platform_id", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki102_computername", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki102_langroup", ""), deferred=True
        ),
        NDRIntField("wki102_ver_major", 0),
        NDRIntField("wki102_ver_minor", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wki102_lanroot", ""), deferred=True
        ),
        NDRIntField("wki102_logged_on_users", 0),
    ]


class LPWKSTA_INFO_502(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("wki502_char_wait", 0),
        NDRIntField("wki502_collection_time", 0),
        NDRIntField("wki502_maximum_collection_count", 0),
        NDRIntField("wki502_keep_conn", 0),
        NDRIntField("wki502_max_cmds", 0),
        NDRIntField("wki502_sess_timeout", 0),
        NDRIntField("wki502_siz_char_buf", 0),
        NDRIntField("wki502_max_threads", 0),
        NDRIntField("wki502_lock_quota", 0),
        NDRIntField("wki502_lock_increment", 0),
        NDRIntField("wki502_lock_maximum", 0),
        NDRIntField("wki502_pipe_increment", 0),
        NDRIntField("wki502_pipe_maximum", 0),
        NDRIntField("wki502_cache_file_timeout", 0),
        NDRIntField("wki502_dormant_file_limit", 0),
        NDRIntField("wki502_read_ahead_throughput", 0),
        NDRIntField("wki502_num_mailslot_buffers", 0),
        NDRIntField("wki502_num_srv_announce_buffers", 0),
        NDRIntField("wki502_max_illegal_datagram_events", 0),
        NDRIntField("wki502_illegal_datagram_event_reset_frequency", 0),
        NDRSignedIntField("wki502_log_election_packets", 0),
        NDRSignedIntField("wki502_use_opportunistic_locking", 0),
        NDRSignedIntField("wki502_use_unlock_behind", 0),
        NDRSignedIntField("wki502_use_close_behind", 0),
        NDRSignedIntField("wki502_buf_named_pipes", 0),
        NDRSignedIntField("wki502_use_lock_read_unlock", 0),
        NDRSignedIntField("wki502_utilize_nt_caching", 0),
        NDRSignedIntField("wki502_use_raw_read", 0),
        NDRSignedIntField("wki502_use_raw_write", 0),
        NDRSignedIntField("wki502_use_write_raw_data", 0),
        NDRSignedIntField("wki502_use_encryption", 0),
        NDRSignedIntField("wki502_buf_files_deny_write", 0),
        NDRSignedIntField("wki502_buf_read_only_files", 0),
        NDRSignedIntField("wki502_force_core_create_mode", 0),
        NDRSignedIntField("wki502_use_512_byte_max_transfer", 0),
    ]


class LPWKSTA_INFO_1013(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("wki1013_keep_conn", 0)]


class LPWKSTA_INFO_1018(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("wki1018_sess_timeout", 0)]


class LPWKSTA_INFO_1046(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("wki1046_dormant_file_limit", 0)]


class NetrWkstaGetInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRIntField("Level", 0),
    ]


class NetrWkstaGetInfo_Response(NDRPacket):
    fields_desc = [
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_100(), LPWKSTA_INFO_100
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 100),
                        (lambda _, val: val.tag == 100),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_101(), LPWKSTA_INFO_101
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 101),
                        (lambda _, val: val.tag == 101),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_102(), LPWKSTA_INFO_102
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 102),
                        (lambda _, val: val.tag == 102),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_502(), LPWKSTA_INFO_502
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 502),
                        (lambda _, val: val.tag == 502),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1013(), LPWKSTA_INFO_1013
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1013),
                        (lambda _, val: val.tag == 1013),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1018(), LPWKSTA_INFO_1018
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1018),
                        (lambda _, val: val.tag == 1018),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1046(), LPWKSTA_INFO_1046
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1046),
                        (lambda _, val: val.tag == 1046),
                    ),
                ),
            ],
            StrFixedLenField("WkstaInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRIntField("status", 0),
    ]


class NetrWkstaSetInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_100(), LPWKSTA_INFO_100
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 100),
                        (lambda _, val: val.tag == 100),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_101(), LPWKSTA_INFO_101
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 101),
                        (lambda _, val: val.tag == 101),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_102(), LPWKSTA_INFO_102
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 102),
                        (lambda _, val: val.tag == 102),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_502(), LPWKSTA_INFO_502
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 502),
                        (lambda _, val: val.tag == 502),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1013(), LPWKSTA_INFO_1013
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1013),
                        (lambda _, val: val.tag == 1013),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1018(), LPWKSTA_INFO_1018
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1018),
                        (lambda _, val: val.tag == 1018),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaInfo", LPWKSTA_INFO_1046(), LPWKSTA_INFO_1046
                        )
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1046),
                        (lambda _, val: val.tag == 1046),
                    ),
                ),
            ],
            StrFixedLenField("WkstaInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
    ]


class NetrWkstaSetInfo_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
        NDRIntField("status", 0),
    ]


class LPWKSTA_USER_INFO_0(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkui0_username", ""), deferred=True
        )
    ]


class LPWKSTA_USER_INFO_0_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPWKSTA_USER_INFO_0()],
                LPWKSTA_USER_INFO_0,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPWKSTA_USER_INFO_1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkui1_username", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkui1_logon_domain", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkui1_oth_domains", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkui1_logon_server", ""), deferred=True
        ),
    ]


class LPWKSTA_USER_INFO_1_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPWKSTA_USER_INFO_1()],
                LPWKSTA_USER_INFO_1,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPWKSTA_USER_ENUM_STRUCT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaUserInfo",
                            LPWKSTA_USER_INFO_0_CONTAINER(),
                            LPWKSTA_USER_INFO_0_CONTAINER,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaUserInfo",
                            LPWKSTA_USER_INFO_1_CONTAINER(),
                            LPWKSTA_USER_INFO_1_CONTAINER,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
            ],
            StrFixedLenField("WkstaUserInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class NetrWkstaUserEnum_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRPacketField(
            "UserInfo", LPWKSTA_USER_ENUM_STRUCT(), LPWKSTA_USER_ENUM_STRUCT
        ),
        NDRIntField("PreferredMaximumLength", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
    ]


class NetrWkstaUserEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "UserInfo", LPWKSTA_USER_ENUM_STRUCT(), LPWKSTA_USER_ENUM_STRUCT
        ),
        NDRIntField("TotalEntries", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
        NDRIntField("status", 0),
    ]


class LPWKSTA_TRANSPORT_INFO_0(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("wkti0_quality_of_service", 0),
        NDRIntField("wkti0_number_of_vcs", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkti0_transport_name", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("wkti0_transport_address", ""), deferred=True
        ),
        NDRIntField("wkti0_wan_ish", 0),
    ]


class LPWKSTA_TRANSPORT_INFO_0_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", None, size_of="Buffer"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "Buffer",
                [LPWKSTA_TRANSPORT_INFO_0()],
                LPWKSTA_TRANSPORT_INFO_0,
                size_is=lambda pkt: pkt.EntriesRead,
            ),
            deferred=True,
        ),
    ]


class LPWKSTA_TRANSPORT_ENUM_STRUCT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "WkstaTransportInfo",
                            LPWKSTA_TRANSPORT_INFO_0_CONTAINER(),
                            LPWKSTA_TRANSPORT_INFO_0_CONTAINER,
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                )
            ],
            StrFixedLenField("WkstaTransportInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class NetrWkstaTransportEnum_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRPacketField(
            "TransportInfo",
            LPWKSTA_TRANSPORT_ENUM_STRUCT(),
            LPWKSTA_TRANSPORT_ENUM_STRUCT,
        ),
        NDRIntField("PreferredMaximumLength", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
    ]


class NetrWkstaTransportEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField(
            "TransportInfo",
            LPWKSTA_TRANSPORT_ENUM_STRUCT(),
            LPWKSTA_TRANSPORT_ENUM_STRUCT,
        ),
        NDRIntField("TotalEntries", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
        NDRIntField("status", 0),
    ]


class NetrWkstaTransportAdd_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRIntField("Level", 0),
        NDRPacketField(
            "TransportInfo", LPWKSTA_TRANSPORT_INFO_0(), LPWKSTA_TRANSPORT_INFO_0
        ),
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
    ]


class NetrWkstaTransportAdd_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
        NDRIntField("status", 0),
    ]


class NetrWkstaTransportDel_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("TransportName", "")),
        NDRIntField("ForceLevel", 0),
    ]


class NetrWkstaTransportDel_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class LPUSE_INFO_0(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui0_local", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui0_remote", ""), deferred=True
        ),
    ]


class LPUSE_INFO_1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_local", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_remote", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_password", ""), deferred=True
        ),
        NDRIntField("ui1_status", 0),
        NDRIntField("ui1_asg_type", 0),
        NDRIntField("ui1_refcount", 0),
        NDRIntField("ui1_usecount", 0),
    ]


class USE_INFO_1(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_local", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_remote", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui1_password", ""), deferred=True
        ),
        NDRIntField("ui1_status", 0),
        NDRIntField("ui1_asg_type", 0),
        NDRIntField("ui1_refcount", 0),
        NDRIntField("ui1_usecount", 0),
    ]


class LPUSE_INFO_2(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("ui2_useinfo", USE_INFO_1(), USE_INFO_1),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui2_username", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui2_domainname", ""), deferred=True
        ),
    ]


class USE_INFO_2(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("ui2_useinfo", USE_INFO_1(), USE_INFO_1),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui2_username", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("ui2_domainname", ""), deferred=True
        ),
    ]


class LPUSE_INFO_3(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRPacketField("ui3_ui2", USE_INFO_2(), USE_INFO_2),
        NDRIntField("ui3_flags", 0),
    ]


class NetrUseAdd_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_0(), LPUSE_INFO_0)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_1(), LPUSE_INFO_1)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_2(), LPUSE_INFO_2)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_3(), LPUSE_INFO_3)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("InfoStruct", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
    ]


class NetrUseAdd_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRIntField("ErrorParameter", 0)),
        NDRIntField("status", 0),
    ]


class NetrUseGetInfo_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("UseName", ""),
        NDRIntField("Level", 0),
    ]


class NetrUseGetInfo_Response(NDRPacket):
    fields_desc = [
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_0(), LPUSE_INFO_0)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_1(), LPUSE_INFO_1)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_2(), LPUSE_INFO_2)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField("InfoStruct", LPUSE_INFO_3(), LPUSE_INFO_3)
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 3),
                        (lambda _, val: val.tag == 3),
                    ),
                ),
            ],
            StrFixedLenField("InfoStruct", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
        NDRIntField("status", 0),
    ]


class NetrUseDel_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("UseName", ""),
        NDRIntField("ForceLevel", 0),
    ]


class NetrUseDel_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class LPUSE_INFO_0_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", 0),
        NDRFullPointerField(
            NDRPacketField("Buffer", LPUSE_INFO_0(), LPUSE_INFO_0), deferred=True
        ),
    ]


class LPUSE_INFO_1_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", 0),
        NDRFullPointerField(
            NDRPacketField("Buffer", LPUSE_INFO_1(), LPUSE_INFO_1), deferred=True
        ),
    ]


class LPUSE_INFO_2_CONTAINER(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntriesRead", 0),
        NDRFullPointerField(
            NDRPacketField("Buffer", LPUSE_INFO_2(), LPUSE_INFO_2), deferred=True
        ),
    ]


class LPUSE_ENUM_STRUCT(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Level", 0),
        NDRUnionField(
            [
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "UseInfo", LPUSE_INFO_0_CONTAINER(), LPUSE_INFO_0_CONTAINER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 0),
                        (lambda _, val: val.tag == 0),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "UseInfo", LPUSE_INFO_1_CONTAINER(), LPUSE_INFO_1_CONTAINER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 1),
                        (lambda _, val: val.tag == 1),
                    ),
                ),
                (
                    NDRFullPointerField(
                        NDRPacketField(
                            "UseInfo", LPUSE_INFO_2_CONTAINER(), LPUSE_INFO_2_CONTAINER
                        ),
                        deferred=True,
                    ),
                    (
                        (lambda pkt: getattr(pkt, "Level", None) == 2),
                        (lambda _, val: val.tag == 2),
                    ),
                ),
            ],
            StrFixedLenField("UseInfo", "", length=0),
            align=(4, 8),
            switch_fmt=("L", "L"),
        ),
    ]


class NetrUseEnum_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRPacketField("InfoStruct", LPUSE_ENUM_STRUCT(), LPUSE_ENUM_STRUCT),
        NDRIntField("PreferredMaximumLength", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
    ]


class NetrUseEnum_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("InfoStruct", LPUSE_ENUM_STRUCT(), LPUSE_ENUM_STRUCT),
        NDRIntField("TotalEntries", 0),
        NDRFullPointerField(NDRIntField("ResumeHandle", 0)),
        NDRIntField("status", 0),
    ]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]


class LPSTAT_WORKSTATION_0(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRPacketField("StatisticsStartTime", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("BytesReceived", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SmbsReceived", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("PagingReadBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("NonPagingReadBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("CacheReadBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("NetworkReadBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("BytesTransmitted", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("SmbsTransmitted", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("PagingWriteBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("NonPagingWriteBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("CacheWriteBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRPacketField("NetworkWriteBytesRequested", LARGE_INTEGER(), LARGE_INTEGER),
        NDRIntField("InitiallyFailedOperations", 0),
        NDRIntField("FailedCompletionOperations", 0),
        NDRIntField("ReadOperations", 0),
        NDRIntField("RandomReadOperations", 0),
        NDRIntField("ReadSmbs", 0),
        NDRIntField("LargeReadSmbs", 0),
        NDRIntField("SmallReadSmbs", 0),
        NDRIntField("WriteOperations", 0),
        NDRIntField("RandomWriteOperations", 0),
        NDRIntField("WriteSmbs", 0),
        NDRIntField("LargeWriteSmbs", 0),
        NDRIntField("SmallWriteSmbs", 0),
        NDRIntField("RawReadsDenied", 0),
        NDRIntField("RawWritesDenied", 0),
        NDRIntField("NetworkErrors", 0),
        NDRIntField("Sessions", 0),
        NDRIntField("FailedSessions", 0),
        NDRIntField("Reconnects", 0),
        NDRIntField("CoreConnects", 0),
        NDRIntField("Lanman20Connects", 0),
        NDRIntField("Lanman21Connects", 0),
        NDRIntField("LanmanNtConnects", 0),
        NDRIntField("ServerDisconnects", 0),
        NDRIntField("HungSessions", 0),
        NDRIntField("UseCount", 0),
        NDRIntField("FailedUseCount", 0),
        NDRIntField("CurrentCommands", 0),
    ]


class NetrWorkstationStatisticsGet_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServiceName", "")),
        NDRIntField("Level", 0),
        NDRIntField("Options", 0),
    ]


class NetrWorkstationStatisticsGet_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("Buffer", LPSTAT_WORKSTATION_0(), LPSTAT_WORKSTATION_0)
        ),
        NDRIntField("status", 0),
    ]


class PNETSETUP_JOIN_STATUS(IntEnum):
    NetSetupUnknownStatus = 0
    NetSetupUnjoined = 1
    NetSetupWorkgroupName = 2
    NetSetupDomainName = 3


class NetrGetJoinInformation_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NameBuffer", "")),
    ]


class NetrGetJoinInformation_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("NameBuffer", "")),
        NDRInt3264EnumField("BufferType", 0, PNETSETUP_JOIN_STATUS),
        NDRIntField("status", 0),
    ]


class PJOINPR_ENCRYPTED_USER_PASSWORD(NDRPacket):
    fields_desc = [StrFixedLenField("Buffer", "", length=((8 + (256 * 2)) + 8))]


class NetrJoinDomain2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("DomainNameParam", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("MachineAccountOU", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrJoinDomain2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrUnjoinDomain2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrUnjoinDomain2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrRenameMachineInDomain2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("MachineName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrRenameMachineInDomain2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NETSETUP_NAME_TYPE(IntEnum):
    NetSetupUnknown = 0
    NetSetupMachine = 1
    NetSetupWorkgroup = 2
    NetSetupDomain = 3
    NetSetupNonExistentDomain = 4
    NetSetupDnsMachine = 5


class NetrValidateName2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("NameToValidate", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRInt3264EnumField("NameType", 0, NETSETUP_NAME_TYPE),
    ]


class NetrValidateName2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrGetJoinableOUs2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("DomainNameParam", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("OUCount", 0),
    ]


class NetrGetJoinableOUs2_Response(NDRPacket):
    fields_desc = [
        NDRIntField("OUCount", None, size_of="OUs"),
        NDRConfVarStrLenFieldUtf16("OUs", "", size_is=lambda pkt: pkt.OUCount),
        NDRIntField("status", 0),
    ]


class NetrAddAlternateComputerName_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AlternateName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrAddAlternateComputerName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrRemoveAlternateComputerName_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AlternateName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrRemoveAlternateComputerName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrSetPrimaryComputerName_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("PrimaryName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrSetPrimaryComputerName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NET_COMPUTER_NAME_TYPE(IntEnum):
    NetPrimaryComputerName = 0
    NetAlternateComputerNames = 1
    NetAllComputerNames = 2
    NetComputerNameTypeMax = 3


class PUNICODE_STRING(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRShortField("Length", None, size_of="Buffer", adjust=lambda _, x: (x * 2)),
        NDRShortField(
            "MaximumLength", None, size_of="Buffer", adjust=lambda _, x: (x * 2)
        ),
        NDRFullPointerField(
            NDRConfVarStrLenFieldUtf16(
                "Buffer",
                "",
                size_is=lambda pkt: (pkt.MaximumLength // 2),
                length_is=lambda pkt: (pkt.Length // 2),
            ),
            deferred=True,
        ),
    ]


class PNET_COMPUTER_NAME_ARRAY(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("EntryCount", None, size_of="ComputerNames"),
        NDRFullPointerField(
            NDRConfPacketListField(
                "ComputerNames",
                [PUNICODE_STRING()],
                PUNICODE_STRING,
                size_is=lambda pkt: pkt.EntryCount,
            ),
            deferred=True,
        ),
    ]


class NetrEnumerateComputerNames_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRInt3264EnumField("NameType", 0, NET_COMPUTER_NAME_TYPE),
        NDRIntField("Reserved", 0),
    ]


class NetrEnumerateComputerNames_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ComputerNames", PNET_COMPUTER_NAME_ARRAY(), PNET_COMPUTER_NAME_ARRAY
            )
        ),
        NDRIntField("status", 0),
    ]


class PJOINPR_ENCRYPTED_USER_PASSWORD_AES(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        StrFixedLenField("AuthData", "", length=64),
        StrFixedLenField("Salt", "", length=16),
        NDRIntField("cbCipher", None, size_of="Cipher"),
        NDRFullPointerField(
            NDRConfStrLenField("Cipher", "", size_is=lambda pkt: pkt.cbCipher),
            deferred=True,
        ),
    ]


class NetrJoinDomain3_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("DomainNameParam", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("MachineAccountOU", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES(),
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrJoinDomain3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrUnjoinDomain3_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES(),
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrUnjoinDomain3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrRenameMachineInDomain3_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("MachineName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Options", 0),
    ]


class NetrRenameMachineInDomain3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrValidateName3_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRConfVarStrNullFieldUtf16("NameToValidate", ""),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AccountName", "")),
        NDRFullPointerField(
            NDRPacketField(
                "Password",
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES(),
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES,
            )
        ),
        NDRInt3264EnumField("NameType", 0, NETSETUP_NAME_TYPE),
    ]


class NetrValidateName3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrAddAlternateComputerName2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AlternateName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES(),
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrAddAlternateComputerName2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrRemoveAlternateComputerName2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("AlternateName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES(),
                PJOINPR_ENCRYPTED_USER_PASSWORD_AES,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrRemoveAlternateComputerName2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class NetrSetPrimaryComputerName2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("ServerName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("PrimaryName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("DomainAccount", "")),
        NDRFullPointerField(
            NDRPacketField(
                "EncryptedPassword",
                PJOINPR_ENCRYPTED_USER_PASSWORD(),
                PJOINPR_ENCRYPTED_USER_PASSWORD,
            )
        ),
        NDRIntField("Reserved", 0),
    ]


class NetrSetPrimaryComputerName2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


WKSSVC_OPNUMS = {
    0: DceRpcOp(NetrWkstaGetInfo_Request, NetrWkstaGetInfo_Response),
    1: DceRpcOp(NetrWkstaSetInfo_Request, NetrWkstaSetInfo_Response),
    2: DceRpcOp(NetrWkstaUserEnum_Request, NetrWkstaUserEnum_Response),
    # 3: Opnum3NotUsedOnWire,
    # 4: Opnum4NotUsedOnWire,
    5: DceRpcOp(NetrWkstaTransportEnum_Request, NetrWkstaTransportEnum_Response),
    6: DceRpcOp(NetrWkstaTransportAdd_Request, NetrWkstaTransportAdd_Response),
    7: DceRpcOp(NetrWkstaTransportDel_Request, NetrWkstaTransportDel_Response),
    8: DceRpcOp(NetrUseAdd_Request, NetrUseAdd_Response),
    9: DceRpcOp(NetrUseGetInfo_Request, NetrUseGetInfo_Response),
    10: DceRpcOp(NetrUseDel_Request, NetrUseDel_Response),
    11: DceRpcOp(NetrUseEnum_Request, NetrUseEnum_Response),
    # 12: Opnum12NotUsedOnWire,
    13: DceRpcOp(
        NetrWorkstationStatisticsGet_Request, NetrWorkstationStatisticsGet_Response
    ),
    # 14: Opnum14NotUsedOnWire,
    # 15: Opnum15NotUsedOnWire,
    # 16: Opnum16NotUsedOnWire,
    # 17: Opnum17NotUsedOnWire,
    # 18: Opnum18NotUsedOnWire,
    # 19: Opnum19NotUsedOnWire,
    20: DceRpcOp(NetrGetJoinInformation_Request, NetrGetJoinInformation_Response),
    # 21: Opnum21NotUsedOnWire,
    22: DceRpcOp(NetrJoinDomain2_Request, NetrJoinDomain2_Response),
    23: DceRpcOp(NetrUnjoinDomain2_Request, NetrUnjoinDomain2_Response),
    24: DceRpcOp(
        NetrRenameMachineInDomain2_Request, NetrRenameMachineInDomain2_Response
    ),
    25: DceRpcOp(NetrValidateName2_Request, NetrValidateName2_Response),
    26: DceRpcOp(NetrGetJoinableOUs2_Request, NetrGetJoinableOUs2_Response),
    27: DceRpcOp(
        NetrAddAlternateComputerName_Request, NetrAddAlternateComputerName_Response
    ),
    28: DceRpcOp(
        NetrRemoveAlternateComputerName_Request,
        NetrRemoveAlternateComputerName_Response,
    ),
    29: DceRpcOp(
        NetrSetPrimaryComputerName_Request, NetrSetPrimaryComputerName_Response
    ),
    30: DceRpcOp(
        NetrEnumerateComputerNames_Request, NetrEnumerateComputerNames_Response
    ),
    31: DceRpcOp(NetrJoinDomain3_Request, NetrJoinDomain3_Response),
    32: DceRpcOp(NetrUnjoinDomain3_Request, NetrUnjoinDomain3_Response),
    33: DceRpcOp(
        NetrRenameMachineInDomain3_Request, NetrRenameMachineInDomain3_Response
    ),
    34: DceRpcOp(NetrValidateName3_Request, NetrValidateName3_Response),
    35: DceRpcOp(
        NetrAddAlternateComputerName2_Request, NetrAddAlternateComputerName2_Response
    ),
    36: DceRpcOp(
        NetrRemoveAlternateComputerName2_Request,
        NetrRemoveAlternateComputerName2_Response,
    ),
    37: DceRpcOp(
        NetrSetPrimaryComputerName2_Request, NetrSetPrimaryComputerName2_Response
    ),
}
register_dcerpc_interface(
    name="wkssvc",
    uuid=uuid.UUID("6BFFD098-A112-3610-9833-46C3F87E345A"),
    version="1.0",
    opnums=WKSSVC_OPNUMS,
)



### ms-csvp.idl
"""
RPC definitions for the following interfaces:
- IClusterStorage2 (v0.0): None
- IClusterStorage3 (v0.0): 11942D87-A1DE-4E7F-83FB-A840D9C5928D
- IClusterNetwork2 (v0.0): None
- IClusterCleanup (v0.0): None
- IClusterSetup (v0.0): None
- IClusterLog (v0.0): None
- IClusterLogEx (v0.0): BD7C23C2-C805-457C-8F86-D17FE6B9D19F
- IClusterFirewall (v0.0): None
- IClusterUpdate (v0.0): None
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField, StrFixedLenFieldUtf16
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrLenField,
    NDRConfVarStrLenFieldUtf16,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    NDRSignedByteField,
    NDRSignedIntField,
    NDRUnionField,
    register_com_interface,
)


class CPREP_DISKID_ENUM(IntEnum):
    CprepIdSignature = 0
    CprepIdGuid = 1
    CprepIdNumber = 4000
    CprepIdUnknown = 5000


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class CPREP_DISKID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRInt3264EnumField("DiskIdType", 0, CPREP_DISKID_ENUM),
        NDRUnionField(
            [
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "DiskIdType", None)
                            == CPREP_DISKID_ENUM.CprepIdSignature
                        ),
                        (lambda _, val: val.tag == CPREP_DISKID_ENUM.CprepIdSignature),
                    ),
                ),
                (
                    NDRPacketField("value", GUID(), GUID),
                    (
                        (
                            lambda pkt: getattr(pkt, "DiskIdType", None)
                            == CPREP_DISKID_ENUM.CprepIdGuid
                        ),
                        (lambda _, val: val.tag == CPREP_DISKID_ENUM.CprepIdGuid),
                    ),
                ),
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "DiskIdType", None)
                            == CPREP_DISKID_ENUM.CprepIdNumber
                        ),
                        (lambda _, val: val.tag == CPREP_DISKID_ENUM.CprepIdNumber),
                    ),
                ),
                (
                    NDRIntField("value", 0),
                    (
                        (
                            lambda pkt: getattr(pkt, "DiskIdType", None)
                            == CPREP_DISKID_ENUM.CprepIdUnknown
                        ),
                        (lambda _, val: val.tag == CPREP_DISKID_ENUM.CprepIdUnknown),
                    ),
                ),
            ],
            StrFixedLenField("value", "", length=0),
            align=(2, 4),
            switch_fmt=("H", "I"),
        ),
    ]


class CprepDiskGetUniqueIds3_Request(NDRPacket):
    fields_desc = [NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID)]


class CprepDiskGetUniqueIds3_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenField("ppbDeviceIdHeader", "", size_is=lambda pkt: pkt.pcbDihSize),
        NDRIntField("pcbDihSize", None, size_of="ppbDeviceIdHeader"),
        NDRConfStrLenField("ppDeviceDescriptor", "", size_is=lambda pkt: pkt.pcbDdSize),
        NDRIntField("pcbDdSize", None, size_of="ppDeviceDescriptor"),
        NDRIntField("status", 0),
    ]


class CprepCheckNetFtBindings3_Request(NDRPacket):
    fields_desc = []


class CprepCheckNetFtBindings3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepCsvTestSetup3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("TestShareGuid", GUID(), GUID),
        NDRConfVarStrNullFieldUtf16("Reserved", ""),
    ]


class CprepCsvTestSetup3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepIsNodeClustered3_Request(NDRPacket):
    fields_desc = []


class CprepIsNodeClustered3_Response(NDRPacket):
    fields_desc = [NDRSignedByteField("pbIsClusterNode", 0), NDRIntField("status", 0)]


class CprepCreateNewSmbShares3_Request(NDRPacket):
    fields_desc = []


class CprepCreateNewSmbShares3_Response(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenFieldUtf16(
            "ppwszSharePaths", "", size_is=lambda pkt: pkt.pdwNumberOfPaths
        ),
        NDRIntField("pdwNumberOfPaths", None, size_of="ppwszSharePaths"),
        NDRIntField("status", 0),
    ]


class CprepConnectToNewSmbShares3_Request(NDRPacket):
    fields_desc = [
        NDRConfVarStrLenFieldUtf16(
            "ppwszSharePaths", "", size_is=lambda pkt: pkt.dwNumberOfPaths
        ),
        NDRIntField("dwNumberOfPaths", None, size_of="ppwszSharePaths"),
    ]


class CprepConnectToNewSmbShares3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DiskStackType(IntEnum):
    DiskStackScsiPort = 0
    DiskStackStorPort = 1
    DiskStackFullPort = 2


class CPREP_SCSI_ADDRESS(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Length", 0),
        NDRByteField("PortNumber", 0),
        NDRByteField("PathId", 0),
        NDRByteField("TargetId", 0),
        NDRByteField("Lun", 0),
    ]


class DISK_PROPS_EX(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("DiskNumber", 0),
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRIntField("DiskBusType", 0),
        NDRInt3264EnumField("StackType", 0, DiskStackType),
        NDRPacketField("ScsiAddress", CPREP_SCSI_ADDRESS(), CPREP_SCSI_ADDRESS),
        NDRSignedIntField("DiskIsClusterable", 0),
        StrFixedLenFieldUtf16("AdapterDesc", "", length=260 * 2),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pwszFriendlyName", ""), deferred=True
        ),
        NDRIntField("NumPaths", 0),
        NDRIntField("Flags", 0),
        NDRIntField("ExtendedFlags", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pwszPoolName", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pwszPage83Id", ""), deferred=True
        ),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("pwszSerialNumber", ""), deferred=True
        ),
        NDRPacketField("guidPoolId", GUID(), GUID),
    ]


class CprepDiskGetProps3_Request(NDRPacket):
    fields_desc = [NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID)]


class CprepDiskGetProps3_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pDiskProps", DISK_PROPS_EX(), DISK_PROPS_EX),
        NDRIntField("status", 0),
    ]


class CprepDiskIsReadOnly3_Request(NDRPacket):
    fields_desc = [NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID)]


class CprepDiskIsReadOnly3_Response(NDRPacket):
    fields_desc = [NDRSignedByteField("pbReadOnly", 0), NDRIntField("status", 0)]


class CprepDiskPRRegister3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("OldPrKey", 0),
        NDRLongField("NewPrKey", 0),
    ]


class CprepDiskPRRegister3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepDiskFindKey3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("Key", 0),
    ]


class CprepDiskFindKey3_Response(NDRPacket):
    fields_desc = [NDRSignedByteField("pbFound", 0), NDRIntField("status", 0)]


class CprepDiskPRPreempt3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("OwnerKey", 0),
        NDRLongField("NewKey", 0),
    ]


class CprepDiskPRPreempt3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepDiskPRReserve3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("Key", 0),
    ]


class CprepDiskPRReserve3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepDiskIsPRPresent3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("Key", 0),
    ]


class CprepDiskIsPRPresent3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepDiskPRRelease3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("Key", 0),
    ]


class CprepDiskPRRelease3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CprepDiskPRClear3_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("DiskId", CPREP_DISKID(), CPREP_DISKID),
        NDRLongField("Key", 0),
    ]


class CprepDiskPRClear3_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


ICLUSTERSTORAGE3_OPNUMS = {
    0: DceRpcOp(CprepDiskGetUniqueIds3_Request, CprepDiskGetUniqueIds3_Response),
    1: DceRpcOp(CprepCheckNetFtBindings3_Request, CprepCheckNetFtBindings3_Response),
    2: DceRpcOp(CprepCsvTestSetup3_Request, CprepCsvTestSetup3_Response),
    3: DceRpcOp(CprepIsNodeClustered3_Request, CprepIsNodeClustered3_Response),
    4: DceRpcOp(CprepCreateNewSmbShares3_Request, CprepCreateNewSmbShares3_Response),
    5: DceRpcOp(
        CprepConnectToNewSmbShares3_Request, CprepConnectToNewSmbShares3_Response
    ),
    6: DceRpcOp(CprepDiskGetProps3_Request, CprepDiskGetProps3_Response),
    7: DceRpcOp(CprepDiskIsReadOnly3_Request, CprepDiskIsReadOnly3_Response),
    8: DceRpcOp(CprepDiskPRRegister3_Request, CprepDiskPRRegister3_Response),
    9: DceRpcOp(CprepDiskFindKey3_Request, CprepDiskFindKey3_Response),
    10: DceRpcOp(CprepDiskPRPreempt3_Request, CprepDiskPRPreempt3_Response),
    11: DceRpcOp(CprepDiskPRReserve3_Request, CprepDiskPRReserve3_Response),
    12: DceRpcOp(CprepDiskIsPRPresent3_Request, CprepDiskIsPRPresent3_Response),
    13: DceRpcOp(CprepDiskPRRelease3_Request, CprepDiskPRRelease3_Response),
    14: DceRpcOp(CprepDiskPRClear3_Request, CprepDiskPRClear3_Response),
}
register_com_interface(
    name="IClusterStorage3",
    uuid=uuid.UUID("11942D87-A1DE-4E7F-83FB-A840D9C5928D"),
    opnums=ICLUSTERSTORAGE3_OPNUMS,
)


class ClusterLogExFlag(IntEnum):
    ClusterLogFlagNone = 0
    ClusterLogFlagLocalTime = 1
    ClusterLogFlagSkipClusterState = 2


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class GenerateClusterLog_Request(NDRPacket):
    fields_desc = [
        NDRIntField("SpanMinutes", 0),
        NDRInt3264EnumField("flags", 0, ClusterLogExFlag),
    ]


class GenerateClusterLog_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("LogFilePath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class GenerateClusterHealthLog_Request(NDRPacket):
    fields_desc = [
        NDRIntField("SpanMinutes", 0),
        NDRInt3264EnumField("flags", 0, ClusterLogExFlag),
    ]


class GenerateClusterHealthLog_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("LogFilePath", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


ICLUSTERLOGEX_OPNUMS = {
    0: DceRpcOp(GenerateClusterLog_Request, GenerateClusterLog_Response),
    1: DceRpcOp(GenerateClusterHealthLog_Request, GenerateClusterHealthLog_Response),
}
register_com_interface(
    name="IClusterLogEx",
    uuid=uuid.UUID("BD7C23C2-C805-457C-8F86-D17FE6B9D19F"),
    opnums=ICLUSTERLOGEX_OPNUMS,
)



### ms-mqqp.idl
"""
RPC definitions for the following interfaces:
- qm2qm (v1.0): 1088a980-eae5-11d0-8d9b-00a02453c337
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRByteField,
    NDRConfVarStrLenField,
    NDRContextHandle,
    NDRFullPointerField,
    NDRInt3264EnumField,
    NDRIntField,
    NDRLongField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class REMOTEREADACK(IntEnum):
    RR_UNKNOWN = 1
    RR_NACK = 2
    RR_ACK = 3


class REMOTEREADDESC(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("hRemoteQueue", 0),
        NDRIntField("hCursor", 0),
        NDRIntField("ulAction", 0),
        NDRIntField("ulTimeout", 0),
        NDRIntField("dwSize", None, size_of="lpBuffer"),
        NDRIntField("dwQueue", 0),
        NDRIntField("dwRequestID", 0),
        NDRIntField("Reserved", 0),
        NDRIntField("dwArriveTime", 0),
        NDRInt3264EnumField("eAckNack", 0, REMOTEREADACK),
        NDRFullPointerField(
            NDRConfVarStrLenField(
                "lpBuffer",
                "",
                size_is=lambda pkt: pkt.dwSize,
                length_is=lambda pkt: pkt.dwSize,
            ),
            deferred=True,
        ),
    ]


class RemoteQMStartReceive_Request(NDRPacket):
    fields_desc = [NDRPacketField("lpRemoteReadDesc", REMOTEREADDESC(), REMOTEREADDESC)]


class RemoteQMStartReceive_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("lpRemoteReadDesc", REMOTEREADDESC(), REMOTEREADDESC),
        NDRIntField("status", 0),
    ]


class RemoteQMEndReceive_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("dwAck", 0),
    ]


class RemoteQMEndReceive_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class RemoteQMOpenQueue_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("pLicGuid", GUID(), GUID),
        NDRIntField("dwMQS", 0),
        NDRIntField("hQueue", 0),
        NDRIntField("pQueue", 0),
        NDRIntField("dwpContext", 0),
    ]


class RemoteQMOpenQueue_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("phContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RemoteQMCloseQueue_Request(NDRPacket):
    fields_desc = [NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle)]


class RemoteQMCloseQueue_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRIntField("status", 0),
    ]


class RemoteQMCloseCursor_Request(NDRPacket):
    fields_desc = [NDRIntField("hQueue", 0), NDRIntField("hCursor", 0)]


class RemoteQMCloseCursor_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoteQMCancelReceive_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hQueue", 0),
        NDRIntField("pQueue", 0),
        NDRIntField("dwRequestID", 0),
    ]


class RemoteQMCancelReceive_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoteQMPurgeQueue_Request(NDRPacket):
    fields_desc = [NDRIntField("hQueue", 0)]


class RemoteQMPurgeQueue_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoteQMGetQMQMServerPort_Request(NDRPacket):
    fields_desc = [NDRIntField("dwPortType", 0)]


class RemoteQMGetQMQMServerPort_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RemoteQmGetVersion_Request(NDRPacket):
    fields_desc = []


class RemoteQmGetVersion_Response(NDRPacket):
    fields_desc = [
        NDRByteField("pMajor", 0),
        NDRByteField("pMinor", 0),
        NDRShortField("pBuildNumber", 0),
        NDRIntField("status", 0),
    ]


class REMOTEREADDESC2(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pRemoteReadDesc", REMOTEREADDESC(), REMOTEREADDESC),
            deferred=True,
        ),
        NDRLongField("SequentialId", 0),
    ]


class RemoteQMStartReceive2_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("lpRemoteReadDesc2", REMOTEREADDESC2(), REMOTEREADDESC2)
    ]


class RemoteQMStartReceive2_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("lpRemoteReadDesc2", REMOTEREADDESC2(), REMOTEREADDESC2),
        NDRIntField("status", 0),
    ]


class RemoteQMStartReceiveByLookupId_Request(NDRPacket):
    fields_desc = [
        NDRLongField("LookupId", 0),
        NDRPacketField("lpRemoteReadDesc2", REMOTEREADDESC2(), REMOTEREADDESC2),
    ]


class RemoteQMStartReceiveByLookupId_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pphContext", NDRContextHandle(), NDRContextHandle),
        NDRPacketField("lpRemoteReadDesc2", REMOTEREADDESC2(), REMOTEREADDESC2),
        NDRIntField("status", 0),
    ]


QM2QM_OPNUMS = {
    0: DceRpcOp(RemoteQMStartReceive_Request, RemoteQMStartReceive_Response),
    1: DceRpcOp(RemoteQMEndReceive_Request, RemoteQMEndReceive_Response),
    2: DceRpcOp(RemoteQMOpenQueue_Request, RemoteQMOpenQueue_Response),
    3: DceRpcOp(RemoteQMCloseQueue_Request, RemoteQMCloseQueue_Response),
    4: DceRpcOp(RemoteQMCloseCursor_Request, RemoteQMCloseCursor_Response),
    5: DceRpcOp(RemoteQMCancelReceive_Request, RemoteQMCancelReceive_Response),
    6: DceRpcOp(RemoteQMPurgeQueue_Request, RemoteQMPurgeQueue_Response),
    7: DceRpcOp(RemoteQMGetQMQMServerPort_Request, RemoteQMGetQMQMServerPort_Response),
    8: DceRpcOp(RemoteQmGetVersion_Request, RemoteQmGetVersion_Response),
    9: DceRpcOp(RemoteQMStartReceive2_Request, RemoteQMStartReceive2_Response),
    10: DceRpcOp(
        RemoteQMStartReceiveByLookupId_Request, RemoteQMStartReceiveByLookupId_Response
    ),
}
register_dcerpc_interface(
    name="qm2qm",
    uuid=uuid.UUID("1088a980-eae5-11d0-8d9b-00a02453c337"),
    version="1.0",
    opnums=QM2QM_OPNUMS,
)



### ms-rdpesc.idl
"""
RPC definitions for the following interfaces:
- type_scard_pack (v1.0): A35AF600-9CF4-11CD-A076-08002B2BD711
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid


from scapy.layers.dcerpc import NDRPacket, register_dcerpc_interface

TYPE_SCARD_PACK_OPNUMS = {}
register_dcerpc_interface(
    name="type_scard_pack",
    uuid=uuid.UUID("A35AF600-9CF4-11CD-A076-08002B2BD711"),
    version="1.0",
    opnums=TYPE_SCARD_PACK_OPNUMS,
)



### ms-rpcl.idl
"""
RPC definitions for the following interfaces:
- LocToLoc (v1.0): e33c0cc4-0482-101a-bc0c-02608c6ba218
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRContextHandle,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRShortField,
    register_dcerpc_interface,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class RPC_VERSION(NDRPacket):
    ALIGNMENT = (2, 2)
    fields_desc = [NDRShortField("MajorVersion", 0), NDRShortField("MinorVersion", 0)]


class RPC_SYNTAX_IDENTIFIER(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRPacketField("SyntaxGUID", GUID(), GUID),
        NDRPacketField("SyntaxVersion", RPC_VERSION(), RPC_VERSION),
    ]


class I_nsi_lookup_begin_Request(NDRPacket):
    fields_desc = [
        NDRIntField("entry_name_syntax", 0),
        NDRConfVarStrNullFieldUtf16("entry_name", ""),
        NDRFullPointerField(
            NDRPacketField(
                "interfaceid", RPC_SYNTAX_IDENTIFIER(), RPC_SYNTAX_IDENTIFIER
            )
        ),
        NDRFullPointerField(
            NDRPacketField("xfersyntax", RPC_SYNTAX_IDENTIFIER(), RPC_SYNTAX_IDENTIFIER)
        ),
        NDRPacketField("obj_uuid", GUID(), GUID),
        NDRIntField("binding_max_count", 0),
        NDRIntField("MaxCacheAge", 0),
    ]


class I_nsi_lookup_begin_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("import_context", NDRContextHandle(), NDRContextHandle),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


class I_nsi_lookup_done_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("import_context", NDRContextHandle(), NDRContextHandle)
    ]


class I_nsi_lookup_done_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("import_context", NDRContextHandle(), NDRContextHandle),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


class NSI_BINDING_T(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("string", ""), deferred=True),
        NDRIntField("entry_name_syntax", 0),
        NDRFullPointerField(
            NDRConfVarStrNullFieldUtf16("entry_name", ""), deferred=True
        ),
    ]


class NSI_BINDING_VECTOR_T(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["binding"]
    fields_desc = [
        NDRIntField("count", None, size_of="binding"),
        NDRConfPacketListField(
            "binding",
            [],
            NSI_BINDING_T,
            size_is=lambda pkt: pkt.count,
            conformant_in_struct=True,
        ),
    ]


class I_nsi_lookup_next_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("import_context", NDRContextHandle(), NDRContextHandle)
    ]


class I_nsi_lookup_next_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "binding_vector", NSI_BINDING_VECTOR_T(), NSI_BINDING_VECTOR_T
            )
        ),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


class NSI_UUID_VECTOR_T(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["uuid"]
    fields_desc = [
        NDRIntField("count", None, size_of="uuid"),
        NDRConfPacketListField(
            "uuid",
            [],
            GUID,
            size_is=lambda pkt: pkt.count,
            conformant_in_struct=True,
            ptr_pack=True,
        ),
    ]


class I_nsi_entry_object_inq_next_Request(NDRPacket):
    fields_desc = [NDRPacketField("InqContext", NDRContextHandle(), NDRContextHandle)]


class I_nsi_entry_object_inq_next_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("uuid_vec", NSI_UUID_VECTOR_T(), NSI_UUID_VECTOR_T)
        ),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


class I_nsi_ping_locator_Request(NDRPacket):
    fields_desc = []


class I_nsi_ping_locator_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class I_nsi_entry_object_inq_done_Request(NDRPacket):
    fields_desc = [NDRPacketField("InqContext", NDRContextHandle(), NDRContextHandle)]


class I_nsi_entry_object_inq_done_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("InqContext", NDRContextHandle(), NDRContextHandle),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


class I_nsi_entry_object_inq_begin_Request(NDRPacket):
    fields_desc = [
        NDRIntField("EntryNameSyntax", 0),
        NDRConfVarStrNullFieldUtf16("EntryName", ""),
    ]


class I_nsi_entry_object_inq_begin_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("InqContext", NDRContextHandle(), NDRContextHandle),
        NDRShortField("status", 0),
        NDRIntField("comm_status", 0),
    ]


LOCTOLOC_OPNUMS = {
    0: DceRpcOp(I_nsi_lookup_begin_Request, I_nsi_lookup_begin_Response),
    1: DceRpcOp(I_nsi_lookup_done_Request, I_nsi_lookup_done_Response),
    2: DceRpcOp(I_nsi_lookup_next_Request, I_nsi_lookup_next_Response),
    3: DceRpcOp(
        I_nsi_entry_object_inq_next_Request, I_nsi_entry_object_inq_next_Response
    ),
    4: DceRpcOp(I_nsi_ping_locator_Request, I_nsi_ping_locator_Response),
    5: DceRpcOp(
        I_nsi_entry_object_inq_done_Request, I_nsi_entry_object_inq_done_Response
    ),
    6: DceRpcOp(
        I_nsi_entry_object_inq_begin_Request, I_nsi_entry_object_inq_begin_Response
    ),
}
register_dcerpc_interface(
    name="LocToLoc",
    uuid=uuid.UUID("e33c0cc4-0482-101a-bc0c-02608c6ba218"),
    version="1.0",
    opnums=LOCTOLOC_OPNUMS,
)



### ms-rpce.idl
"""
RPC definitions for the following interfaces:
-
This file is auto-generated by midl-to-scapy, do not modify.
"""

import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    NDRIntField,
    NDRShortField,
    NDRSignedLongField,
)


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class FILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class LARGE_INTEGER(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("QuadPart", 0)]



### ms-imsa.idl
"""
RPC definitions for the following interfaces:
- IMSAdminBaseW (v0.0): 70B51430-B6CA-11d0-B9B9-00A0C922E750
- IMSAdminBase2W (v0.0): 8298d101-f992-43b7-8eca-5052d885b995
- IMSAdminBase3W (v0.0): f612954d-3b0b-4c56-9563-227b7be624b4
- IWamAdmin (v0.0): 29822AB7-F302-11D0-9953-00C04FD919C1
- IWamAdmin2 (v0.0): 29822AB8-F302-11D0-9953-00C04FD919C1
- IIISApplicationAdmin (v0.0): 7C4E1804-E342-483D-A43E-A850CFCC8D18
- IIISCertObj (v0.0): BD0C73BC-805B-4043-9C30-9A28D64DD7D2
This file is auto-generated by midl-to-scapy, do not modify.
"""

from enum import IntEnum
import uuid

from scapy.fields import StrFixedLenField
from scapy.layers.dcerpc import (
    NDRPacket,
    DceRpcOp,
    NDRConfPacketListField,
    NDRConfStrLenField,
    NDRConfStrLenFieldUtf16,
    NDRConfVarStrLenField,
    NDRConfVarStrNullField,
    NDRConfVarStrNullFieldUtf16,
    NDRFullPointerField,
    NDRIntField,
    NDRPacketField,
    NDRRecursiveField,
    NDRRefEmbPointerField,
    NDRShortField,
    NDRSignedByteField,
    NDRSignedIntField,
    NDRSignedLongField,
    NDRSignedShortField,
    register_com_interface,
    register_dcerpc_interface,
)


class AddKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
    ]


class AddKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
    ]


class DeleteKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteChildKeys_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
    ]


class DeleteChildKeys_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumKeys_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDEnumObjectIndex", 0),
    ]


class EnumKeys_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszMDName", "", size_is=lambda pkt: pkt.ADMINDATA_MAX_NAME_LEN
        ),
        NDRIntField("status", 0),
    ]


class CopyKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDSourceHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDSourcePath", "")),
        NDRIntField("hMDDestHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDDestPath", "")),
        NDRSignedIntField("bMDOverwriteFlag", 0),
        NDRSignedIntField("bMDCopyFlag", 0),
    ]


class CopyKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RenameKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDNewName", "")),
    ]


class RenameKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class METADATA_RECORD(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("dwMDIdentifier", 0),
        NDRIntField("dwMDAttributes", 0),
        NDRIntField("dwMDUserType", 0),
        NDRIntField("dwMDDataType", 0),
        NDRIntField("dwMDDataLen", None, size_of="pbMDData"),
        NDRFullPointerField(
            NDRConfStrLenField("pbMDData", "", size_is=lambda pkt: pkt.dwMDDataLen),
            deferred=True,
        ),
        NDRIntField("dwMDDataTag", 0),
    ]


class R_SetData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRPacketField("pmdrMDData", METADATA_RECORD(), METADATA_RECORD),
    ]


class R_SetData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IIS_CRYPTO_BLOB(NDRPacket):
    ALIGNMENT = (4, 4)
    DEPORTED_CONFORMANTS = ["BlobData"]
    fields_desc = [
        NDRIntField("BlobSignature", 0),
        NDRIntField("BlobDataLength", None, size_of="BlobData"),
        NDRConfStrLenField(
            "BlobData",
            "",
            size_is=lambda pkt: pkt.BlobDataLength,
            conformant_in_struct=True,
        ),
    ]


class R_GetData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRPacketField("pmdrMDData", METADATA_RECORD(), METADATA_RECORD),
    ]


class R_GetData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pmdrMDData", METADATA_RECORD(), METADATA_RECORD),
        NDRIntField("pdwMDRequiredDataLen", 0),
        NDRFullPointerField(
            NDRPacketField("ppDataBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class DeleteData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDIdentifier", 0),
        NDRIntField("dwMDDataType", 0),
    ]


class DeleteData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class R_EnumData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRPacketField("pmdrMDData", METADATA_RECORD(), METADATA_RECORD),
        NDRIntField("dwMDEnumDataIndex", 0),
    ]


class R_EnumData_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pmdrMDData", METADATA_RECORD(), METADATA_RECORD),
        NDRIntField("pdwMDRequiredDataLen", 0),
        NDRFullPointerField(
            NDRPacketField("ppDataBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class R_GetAllData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDAttributes", 0),
        NDRIntField("dwMDUserType", 0),
        NDRIntField("dwMDDataType", 0),
        NDRIntField("dwMDBufferSize", 0),
    ]


class R_GetAllData_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pdwMDNumDataEntries", 0),
        NDRIntField("pdwMDDataSetNumber", 0),
        NDRIntField("pdwMDRequiredBufferSize", 0),
        NDRFullPointerField(
            NDRPacketField("ppDataBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class DeleteAllData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDUserType", 0),
        NDRIntField("dwMDDataType", 0),
    ]


class DeleteAllData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CopyData_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDSourceHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDSourcePath", "")),
        NDRIntField("hMDDestHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDDestPath", "")),
        NDRIntField("dwMDAttributes", 0),
        NDRIntField("dwMDUserType", 0),
        NDRIntField("dwMDDataType", 0),
        NDRSignedIntField("bMDCopyFlag", 0),
    ]


class CopyData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetDataPaths_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDIdentifier", 0),
        NDRIntField("dwMDDataType", 0),
        NDRIntField("dwMDBufferSize", 0),
    ]


class GetDataPaths_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszBuffer", "", size_is=lambda pkt: pkt.dwMDBufferSize
        ),
        NDRIntField("pdwMDRequiredBufferSize", 0),
        NDRIntField("status", 0),
    ]


class OpenKey_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("dwMDAccessRequested", 0),
        NDRIntField("dwMDTimeOut", 0),
    ]


class OpenKey_Response(NDRPacket):
    fields_desc = [NDRIntField("phMDNewHandle", 0), NDRIntField("status", 0)]


class CloseKey_Request(NDRPacket):
    fields_desc = [NDRIntField("hMDHandle", 0)]


class CloseKey_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ChangePermissions_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRIntField("dwMDTimeOut", 0),
        NDRIntField("dwMDAccessRequested", 0),
    ]


class ChangePermissions_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class SaveData_Request(NDRPacket):
    fields_desc = []


class SaveData_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class METADATA_HANDLE_INFO(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("dwMDPermissions", 0),
        NDRIntField("dwMDSystemChangeNumber", 0),
    ]


class GetHandleInfo_Request(NDRPacket):
    fields_desc = [NDRIntField("hMDHandle", 0)]


class GetHandleInfo_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pmdhiInfo", METADATA_HANDLE_INFO(), METADATA_HANDLE_INFO),
        NDRIntField("status", 0),
    ]


class GetSystemChangeNumber_Request(NDRPacket):
    fields_desc = []


class GetSystemChangeNumber_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwSystemChangeNumber", 0), NDRIntField("status", 0)]


class GetDataSetNumber_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
    ]


class GetDataSetNumber_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwMDDataSetNumber", 0), NDRIntField("status", 0)]


class PFILETIME(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [NDRIntField("dwLowDateTime", 0), NDRIntField("dwHighDateTime", 0)]


class SetLastChangeTime_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRPacketField("pftMDLastChangeTime", PFILETIME(), PFILETIME),
        NDRSignedIntField("bLocalTime", 0),
    ]


class SetLastChangeTime_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetLastChangeTime_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRSignedIntField("bLocalTime", 0),
    ]


class GetLastChangeTime_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pftMDLastChangeTime", PFILETIME(), PFILETIME),
        NDRIntField("status", 0),
    ]


class R_KeyExchangePhase1_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "pClientKeyExchangeKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField(
                "pClientSignatureKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB
            )
        ),
    ]


class R_KeyExchangePhase1_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField(
                "ppServerKeyExchangeKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField(
                "ppServerSignatureKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB
            )
        ),
        NDRFullPointerField(
            NDRPacketField("ppServerSessionKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class R_KeyExchangePhase2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("pClientSessionKeyBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRFullPointerField(
            NDRPacketField("pClientHashBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
    ]


class R_KeyExchangePhase2_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("ppServerHashBlob", IIS_CRYPTO_BLOB(), IIS_CRYPTO_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class Backup_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDBackupName", "")),
        NDRIntField("dwMDVersion", 0),
        NDRIntField("dwMDFlags", 0),
    ]


class Backup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Restore_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDBackupName", "")),
        NDRIntField("dwMDVersion", 0),
        NDRIntField("dwMDFlags", 0),
    ]


class Restore_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumBackups_Request(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszMDBackupName", "", size_is=lambda pkt: pkt.MD_BACKUP_MAX_LEN
        ),
        NDRIntField("dwMDEnumIndex", 0),
    ]


class EnumBackups_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszMDBackupName", "", size_is=lambda pkt: pkt.MD_BACKUP_MAX_LEN
        ),
        NDRIntField("pdwMDVersion", 0),
        NDRPacketField("pftMDBackupTime", PFILETIME(), PFILETIME),
        NDRIntField("status", 0),
    ]


class DeleteBackup_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDBackupName", "")),
        NDRIntField("dwMDVersion", 0),
    ]


class DeleteBackup_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class MInterfacePointer(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["abData"]
    fields_desc = [
        NDRIntField("ulCntData", None, size_of="abData"),
        NDRConfStrLenField(
            "abData", "", size_is=lambda pkt: pkt.ulCntData, conformant_in_struct=True
        ),
    ]


class UnmarshalInterface_Request(NDRPacket):
    fields_desc = []


class UnmarshalInterface_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("piadmbwInterface", MInterfacePointer(), MInterfacePointer)
        ),
        NDRIntField("status", 0),
    ]


class GUID(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("Data1", 0),
        NDRShortField("Data2", 0),
        NDRShortField("Data3", 0),
        StrFixedLenField("Data4", "", length=8),
    ]


class R_GetServerGuid_Request(NDRPacket):
    fields_desc = []


class R_GetServerGuid_Response(NDRPacket):
    fields_desc = [
        NDRPacketField("pServerGuid", GUID(), GUID),
        NDRIntField("status", 0),
    ]


IMSADMINBASEW_OPNUMS = {
    0: DceRpcOp(AddKey_Request, AddKey_Response),
    1: DceRpcOp(DeleteKey_Request, DeleteKey_Response),
    2: DceRpcOp(DeleteChildKeys_Request, DeleteChildKeys_Response),
    3: DceRpcOp(EnumKeys_Request, EnumKeys_Response),
    4: DceRpcOp(CopyKey_Request, CopyKey_Response),
    5: DceRpcOp(RenameKey_Request, RenameKey_Response),
    6: DceRpcOp(R_SetData_Request, R_SetData_Response),
    7: DceRpcOp(R_GetData_Request, R_GetData_Response),
    8: DceRpcOp(DeleteData_Request, DeleteData_Response),
    9: DceRpcOp(R_EnumData_Request, R_EnumData_Response),
    10: DceRpcOp(R_GetAllData_Request, R_GetAllData_Response),
    11: DceRpcOp(DeleteAllData_Request, DeleteAllData_Response),
    12: DceRpcOp(CopyData_Request, CopyData_Response),
    13: DceRpcOp(GetDataPaths_Request, GetDataPaths_Response),
    14: DceRpcOp(OpenKey_Request, OpenKey_Response),
    15: DceRpcOp(CloseKey_Request, CloseKey_Response),
    16: DceRpcOp(ChangePermissions_Request, ChangePermissions_Response),
    17: DceRpcOp(SaveData_Request, SaveData_Response),
    18: DceRpcOp(GetHandleInfo_Request, GetHandleInfo_Response),
    19: DceRpcOp(GetSystemChangeNumber_Request, GetSystemChangeNumber_Response),
    20: DceRpcOp(GetDataSetNumber_Request, GetDataSetNumber_Response),
    21: DceRpcOp(SetLastChangeTime_Request, SetLastChangeTime_Response),
    22: DceRpcOp(GetLastChangeTime_Request, GetLastChangeTime_Response),
    23: DceRpcOp(R_KeyExchangePhase1_Request, R_KeyExchangePhase1_Response),
    24: DceRpcOp(R_KeyExchangePhase2_Request, R_KeyExchangePhase2_Response),
    25: DceRpcOp(Backup_Request, Backup_Response),
    26: DceRpcOp(Restore_Request, Restore_Response),
    27: DceRpcOp(EnumBackups_Request, EnumBackups_Response),
    28: DceRpcOp(DeleteBackup_Request, DeleteBackup_Response),
    29: DceRpcOp(UnmarshalInterface_Request, UnmarshalInterface_Response),
    30: DceRpcOp(R_GetServerGuid_Request, R_GetServerGuid_Response),
}
register_com_interface(
    name="IMSAdminBaseW",
    uuid=uuid.UUID("70B51430-B6CA-11d0-B9B9-00A0C922E750"),
    opnums=IMSADMINBASEW_OPNUMS,
)


class BackupWithPasswd_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDBackupName", "")),
        NDRIntField("dwMDVersion", 0),
        NDRIntField("dwMDFlags", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszPasswd", "")),
    ]


class BackupWithPasswd_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RestoreWithPasswd_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDBackupName", "")),
        NDRIntField("dwMDVersion", 0),
        NDRIntField("dwMDFlags", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszPasswd", "")),
    ]


class RestoreWithPasswd_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Export_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszPasswd", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszFileName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszSourcePath", "")),
        NDRIntField("dwMDFlags", 0),
    ]


class Export_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class Import_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszPasswd", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszFileName", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszSourcePath", "")),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszDestPath", "")),
        NDRIntField("dwMDFlags", 0),
    ]


class Import_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class RestoreHistory_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDHistoryLocation", "")),
        NDRIntField("dwMDMajorVersion", 0),
        NDRIntField("dwMDMinorVersion", 0),
        NDRIntField("dwMDFlags", 0),
    ]


class RestoreHistory_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class EnumHistory_Request(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszMDHistoryLocation", "", size_is=lambda pkt: pkt.MD_BACKUP_MAX_LEN
        ),
        NDRIntField("dwMDEnumIndex", 0),
    ]


class EnumHistory_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszMDHistoryLocation", "", size_is=lambda pkt: pkt.MD_BACKUP_MAX_LEN
        ),
        NDRIntField("pdwMDMajorVersion", 0),
        NDRIntField("pdwMDMinorVersion", 0),
        NDRPacketField("pftMDHistoryTime", PFILETIME(), PFILETIME),
        NDRIntField("status", 0),
    ]


IMSADMINBASE2W_OPNUMS = {
    0: DceRpcOp(BackupWithPasswd_Request, BackupWithPasswd_Response),
    1: DceRpcOp(RestoreWithPasswd_Request, RestoreWithPasswd_Response),
    2: DceRpcOp(Export_Request, Export_Response),
    3: DceRpcOp(Import_Request, Import_Response),
    4: DceRpcOp(RestoreHistory_Request, RestoreHistory_Response),
    5: DceRpcOp(EnumHistory_Request, EnumHistory_Response),
}
register_com_interface(
    name="IMSAdminBase2W",
    uuid=uuid.UUID("8298d101-f992-43b7-8eca-5052d885b995"),
    opnums=IMSADMINBASE2W_OPNUMS,
)


class GetChildPaths_Request(NDRPacket):
    fields_desc = [
        NDRIntField("hMDHandle", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("pszMDPath", "")),
        NDRIntField("cchMDBufferSize", None, size_of="pszBuffer"),
        NDRConfStrLenFieldUtf16(
            "pszBuffer", "", size_is=lambda pkt: pkt.cchMDBufferSize
        ),
        NDRFullPointerField(NDRIntField("pcchMDRequiredBufferSize", 0)),
    ]


class GetChildPaths_Response(NDRPacket):
    fields_desc = [
        NDRConfStrLenFieldUtf16(
            "pszBuffer", "", size_is=lambda pkt: pkt.cchMDBufferSize
        ),
        NDRFullPointerField(NDRIntField("pcchMDRequiredBufferSize", 0)),
        NDRIntField("status", 0),
    ]


IMSADMINBASE3W_OPNUMS = {0: DceRpcOp(GetChildPaths_Request, GetChildPaths_Response)}
register_com_interface(
    name="IMSAdminBase3W",
    uuid=uuid.UUID("f612954d-3b0b-4c56-9563-227b7be624b4"),
    opnums=IMSADMINBASE3W_OPNUMS,
)


class AppCreate_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fInProc", 0),
    ]


class AppCreate_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AppDelete_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fRecursive", 0),
    ]


class AppDelete_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AppUnLoad_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fRecursive", 0),
    ]


class AppUnLoad_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AppGetStatus_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", ""))]


class AppGetStatus_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwAppStatus", 0), NDRIntField("status", 0)]


class AppDeleteRecoverable_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fRecursive", 0),
    ]


class AppDeleteRecoverable_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class AppRecover_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fRecursive", 0),
    ]


class AppRecover_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWAMADMIN_OPNUMS = {
    0: DceRpcOp(AppCreate_Request, AppCreate_Response),
    1: DceRpcOp(AppDelete_Request, AppDelete_Response),
    2: DceRpcOp(AppUnLoad_Request, AppUnLoad_Response),
    3: DceRpcOp(AppGetStatus_Request, AppGetStatus_Response),
    4: DceRpcOp(AppDeleteRecoverable_Request, AppDeleteRecoverable_Response),
    5: DceRpcOp(AppRecover_Request, AppRecover_Response),
}
register_dcerpc_interface(
    name="IWamAdmin",
    uuid=uuid.UUID("29822AB7-F302-11D0-9953-00C04FD919C1"),
    version="0.0",
    opnums=IWAMADMIN_OPNUMS,
)


class AppCreate2_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRIntField("dwAppMode", 0),
    ]


class AppCreate2_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


IWAMADMIN2_OPNUMS = {0: DceRpcOp(AppCreate2_Request, AppCreate2_Response)}
register_dcerpc_interface(
    name="IWamAdmin2",
    uuid=uuid.UUID("29822AB8-F302-11D0-9953-00C04FD919C1"),
    version="0.0",
    opnums=IWAMADMIN2_OPNUMS,
)


class CreateApplication_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRIntField("dwAppMode", 0),
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szAppPoolId", "")),
        NDRSignedIntField("fCreatePool", 0),
    ]


class CreateApplication_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteApplication_Request(NDRPacket):
    fields_desc = [
        NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szMDPath", "")),
        NDRSignedIntField("fRecursive", 0),
    ]


class DeleteApplication_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class CreateApplicationPool_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szPool", ""))]


class CreateApplicationPool_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class DeleteApplicationPool_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szPool", ""))]


class DeleteApplicationPool_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class FLAGGED_WORD_BLOB(NDRPacket):
    ALIGNMENT = (4, 8)
    DEPORTED_CONFORMANTS = ["asData"]
    fields_desc = [
        NDRIntField("cBytes", 0),
        NDRIntField("clSize", None, size_of="asData"),
        NDRConfStrLenFieldUtf16(
            "asData", "", size_is=lambda pkt: pkt.clSize, conformant_in_struct=True
        ),
    ]


class EnumerateApplicationsInPool_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szPool", ""))]


class EnumerateApplicationsInPool_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("bstrBuffer", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)
        ),
        NDRIntField("status", 0),
    ]


class RecycleApplicationPool_Request(NDRPacket):
    fields_desc = [NDRFullPointerField(NDRConfVarStrNullFieldUtf16("szPool", ""))]


class RecycleApplicationPool_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class GetProcessMode_Request(NDRPacket):
    fields_desc = []


class GetProcessMode_Response(NDRPacket):
    fields_desc = [NDRIntField("pdwMode", 0), NDRIntField("status", 0)]


IIISAPPLICATIONADMIN_OPNUMS = {
    0: DceRpcOp(CreateApplication_Request, CreateApplication_Response),
    1: DceRpcOp(DeleteApplication_Request, DeleteApplication_Response),
    2: DceRpcOp(CreateApplicationPool_Request, CreateApplicationPool_Response),
    3: DceRpcOp(DeleteApplicationPool_Request, DeleteApplicationPool_Response),
    4: DceRpcOp(
        EnumerateApplicationsInPool_Request, EnumerateApplicationsInPool_Response
    ),
    5: DceRpcOp(RecycleApplicationPool_Request, RecycleApplicationPool_Response),
    6: DceRpcOp(GetProcessMode_Request, GetProcessMode_Response),
}
register_dcerpc_interface(
    name="IIISApplicationAdmin",
    uuid=uuid.UUID("7C4E1804-E342-483D-A43E-A850CFCC8D18"),
    version="0.0",
    opnums=IIISAPPLICATIONADMIN_OPNUMS,
)


class put_Opnum7NotUsedOnWire_Request(NDRPacket):
    fields_desc = []


class put_Opnum7NotUsedOnWire_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class put_Opnum8NotUsedOnWire_Request(NDRPacket):
    fields_desc = []


class put_Opnum8NotUsedOnWire_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class put_Opnum9NotUsedOnWire_Request(NDRPacket):
    fields_desc = []


class put_Opnum9NotUsedOnWire_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class put_InstanceName_Request(NDRPacket):
    fields_desc = [NDRPacketField("newVal", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB)]


class put_InstanceName_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class IsInstalledRemote_Request(NDRPacket):
    fields_desc = []


class IsInstalledRemote_Response(NDRPacket):
    fields_desc = [NDRSignedShortField("retval", 0), NDRIntField("status", 0)]


class IsExportableRemote_Request(NDRPacket):
    fields_desc = []


class IsExportableRemote_Response(NDRPacket):
    fields_desc = [NDRSignedShortField("retval", 0), NDRIntField("status", 0)]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class CURRENCY(NDRPacket):
    ALIGNMENT = (8, 8)
    fields_desc = [NDRSignedLongField("int64", 0)]


class VARENUM(IntEnum):
    VT_EMPTY = 0
    VT_NULL = 1
    VT_I2 = 2
    VT_I4 = 3
    VT_R4 = 4
    VT_R8 = 5
    VT_CY = 6
    VT_DATE = 7
    VT_BSTR = 8
    VT_DISPATCH = 9
    VT_ERROR = 10
    VT_BOOL = 11
    VT_VARIANT = 12
    VT_UNKNOWN = 13
    VT_DECIMAL = 14
    VT_I1 = 16
    VT_UI1 = 17
    VT_UI2 = 18
    VT_UI4 = 19
    VT_I8 = 20
    VT_UI8 = 21
    VT_INT = 22
    VT_UINT = 23
    VT_VOID = 24
    VT_HRESULT = 25
    VT_PTR = 26
    VT_SAFEARRAY = 27
    VT_CARRAY = 28
    VT_USERDEFINED = 29
    VT_LPSTR = 30
    VT_LPWSTR = 31
    VT_RECORD = 36
    VT_INT_PTR = 37
    VT_UINT_PTR = 38
    VT_ARRAY = 8192
    VT_BYREF = 16384


class SF_TYPE(IntEnum):
    SF_ERROR = VARENUM.VT_ERROR
    SF_I1 = VARENUM.VT_I1
    SF_I2 = VARENUM.VT_I2
    SF_I4 = VARENUM.VT_I4
    SF_I8 = VARENUM.VT_I8
    SF_BSTR = VARENUM.VT_BSTR
    SF_UNKNOWN = VARENUM.VT_UNKNOWN
    SF_DISPATCH = VARENUM.VT_DISPATCH
    SF_VARIANT = VARENUM.VT_VARIANT
    SF_RECORD = VARENUM.VT_RECORD
    SF_HAVEIID = VARENUM.VT_UNKNOWN | 32768


class SAFEARR_BSTR(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="aBstr"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "aBstr", [], FLAGGED_WORD_BLOB, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_UNKNOWN(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apUnknown"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apUnknown", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class SAFEARR_DISPATCH(NDRPacket):
    ALIGNMENT = (4, 8)
    fields_desc = [
        NDRIntField("Size", None, size_of="apDispatch"),
        NDRRefEmbPointerField(
            NDRConfPacketListField(
                "apDispatch", [], MInterfacePointer, size_is=lambda pkt: pkt.Size
            ),
            deferred=True,
        ),
    ]


class wireVARIANTStr(NDRPacket):
    ALIGNMENT = (4, 4)
    fields_desc = [
        NDRIntField("clSize", 0),
        NDRIntField("rpcReserved", 0),
        NDRShortField("vt", 0),
        NDRShortField("wReserved1", 0),
        NDRShortField("wReserved2", 0),
        NDRShortField("wReserved3", 0),
        NDRRecursiveField("_varUnion"),
    ]


class GetCertInfoRemote_Request(NDRPacket):
    fields_desc = []


class GetCertInfoRemote_Response(NDRPacket):
    fields_desc = [
        NDRFullPointerField(
            NDRPacketField("BinaryVariant", wireVARIANTStr(), wireVARIANTStr)
        ),
        NDRIntField("status", 0),
    ]


class ImportFromBlob_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("InstanceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("Password", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedShortField("bInstallToMetabase", 0),
        NDRSignedShortField("bAllowExport", 0),
        NDRSignedShortField("bOverWriteExisting", 0),
        NDRIntField("cbSize", None, size_of="pBlobBinary"),
        NDRConfVarStrLenField("pBlobBinary", "", size_is=lambda pkt: pkt.cbSize),
    ]


class ImportFromBlob_Response(NDRPacket):
    fields_desc = [NDRIntField("status", 0)]


class ImportFromBlobGetHash_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("InstanceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("Password", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedShortField("bInstallToMetabase", 0),
        NDRSignedShortField("bAllowExport", 0),
        NDRSignedShortField("bOverWriteExisting", 0),
        NDRIntField("cbSize", None, size_of="pBlobBinary"),
        NDRConfVarStrLenField("pBlobBinary", "", size_is=lambda pkt: pkt.cbSize),
    ]


class ImportFromBlobGetHash_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcbCertHashSize", 0),
        NDRFullPointerField(NDRSignedByteField("pCertHash", 0)),
        NDRIntField("status", 0),
    ]


class ExportToBlob_Request(NDRPacket):
    fields_desc = [
        NDRPacketField("InstanceName", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRPacketField("Password", FLAGGED_WORD_BLOB(), FLAGGED_WORD_BLOB),
        NDRSignedShortField("bPrivateKey", 0),
        NDRSignedShortField("bCertChain", 0),
        NDRIntField("pcbSize", 0),
        NDRFullPointerField(NDRConfVarStrNullField("pBlobBinary", "")),
    ]


class ExportToBlob_Response(NDRPacket):
    fields_desc = [
        NDRIntField("pcbSize", 0),
        NDRFullPointerField(NDRConfVarStrNullField("pBlobBinary", "")),
        NDRIntField("status", 0),
    ]


IIISCERTOBJ_OPNUMS = {
    0: DceRpcOp(put_Opnum7NotUsedOnWire_Request, put_Opnum7NotUsedOnWire_Response),
    1: DceRpcOp(put_Opnum8NotUsedOnWire_Request, put_Opnum8NotUsedOnWire_Response),
    2: DceRpcOp(put_Opnum9NotUsedOnWire_Request, put_Opnum9NotUsedOnWire_Response),
    3: DceRpcOp(put_InstanceName_Request, put_InstanceName_Response),
    # 4: Opnum11NotUsedOnWire,
    5: DceRpcOp(IsInstalledRemote_Request, IsInstalledRemote_Response),
    # 6: Opnum13NotUsedOnWire,
    7: DceRpcOp(IsExportableRemote_Request, IsExportableRemote_Response),
    # 8: Opnum15NotUsedOnWire,
    9: DceRpcOp(GetCertInfoRemote_Request, GetCertInfoRemote_Response),
    # 10: Opnum17NotUsedOnWire,
    # 11: Opnum18NotUsedOnWire,
    # 12: Opnum19NotUsedOnWire,
    # 13: Opnum20NotUsedOnWire,
    # 14: Opnum21NotUsedOnWire,
    15: DceRpcOp(ImportFromBlob_Request, ImportFromBlob_Response),
    16: DceRpcOp(ImportFromBlobGetHash_Request, ImportFromBlobGetHash_Response),
    # 17: Opnum24NotUsedOnWire,
    18: DceRpcOp(ExportToBlob_Request, ExportToBlob_Response),
}
register_com_interface(
    name="IIISCertObj",
    uuid=uuid.UUID("BD0C73BC-805B-4043-9C30-9A28D64DD7D2"),
    opnums=IIISCERTOBJ_OPNUMS,
)
